import datetime
import unittest

from minimock import Mock, mock, TraceTracker

from mediamosa.resources import Asset, MediaMosaResource
from mediamosa.api import MediaMosaAPI


class TestMediaMosaResource(unittest.TestCase):
    """Tests the generic functionality of a MediaMosaResource. This is
    done using the Asset object as concrete implementation.
    """

    def setUp(self):
        self.api = MediaMosaAPI('http://video.example.com')
        self.tt = TraceTracker()
        mock('self.api.session', tracker=self.tt)
        self.response = Mock('requests.Response')
        self.response.status_code = 200
        self.api.session.get.mock_returns = self.response

        self.item_dict = {
            u'provider_id': '', u'is_unappropriate': u'FALSE',
            u'videotimestampmodified': u'2012-07-05 11:34:35',
            u'app_id': u'2', u'is_favorite': u'FALSE',
            u'has_streamable_mediafiles': u'TRUE', u'viewed': u'4',
            u'asset_id': u'g1QkoSmSeHdWfGkMKlOlldLn',
            u'ega_still_url': '', u'granted': u'TRUE', u'played': u'1',
            u'mediafile_duration': u'00:00:52.20',
            # u'videotimestamp': u'2012-07-05 11:34:01', # cleared it for partial
            u'vpx_still_url': u'http://filvideod.ugent.be/media/17/Z/Z14cWALWKmfTRjTUKhhQQLv2.jpeg',
            u'owner_id': u'krkeppen', u'is_empty_asset': u'FALSE',
            u'play_restriction_end': '', u'asset_property_is_hidden': u'FALSE',
            u'dublin_core': {u'publisher': u'Kristof Keppens',
            u'rights': u'Kristof Keppens', u'description': u'test',
            u'language': u'nl', u'creator': u'Kristof Keppens',
            u'format': u'streaming video', u'coverage_spatial': '',
            u'date': '', u'relation': '', u'source': u'ugent',
            u'contributor': u', ', u'title': u'test sintel 2',
            u'identifier': '', u'type': u'video', u'coverage_temporal': '',
            u'subject': u'test'}, u'reference_id': '', u'isprivate': u'TRUE',
            u'qualified_dublin_core': {u'isformatof': '',
            u'description_abstract': '', u'license': '', u'created': '',
            u'issued': '', u'rightsholder': '', u'hasformat': '',
            u'title_alternative': '', u'format_medium': '',
            u'format_extent': '', u'isreferencedby': ''},
            u'mediafile_container_type': u'matroska;webm',
            u'is_unappropiate': u'FALSE', u'is_external': u'FALSE',
            u'is_protected': u'FALSE', u'play_restriction_start': '',
            u'group_id': ''}

    # _mmmeta.state

    def test_create_empty_asset(self):
        """Tests if an empty asset can be created"""
        a = Asset('g1QkoSmSeHdWfGkMKlOlldLn')
        self.assertEqual(a._mmmeta.state, MediaMosaResource.STATE.EMPTY)

    def test_create_partial_asset(self):
        """Tests if a partially pre-filled asset can be created"""
        a = Asset.fromdict(self.item_dict)
        self.assertEqual(a._mmmeta.state, MediaMosaResource.STATE.PARTIAL)

    def test_create_full_asset(self):
        """Tests if an fully pre-filled asset can be created"""
        a = Asset.fromdict(self.item_dict, full=True)
        self.assertEqual(a._mmmeta.state, MediaMosaResource.STATE.FULL)

    # _mmmeta.api

    def test_create_connected_asset(self):
        """Tests if a connected asset can be created"""
        a = Asset('g1QkoSmSeHdWfGkMKlOlldLn', api=self.api)
        self.assertTrue(a.is_connected())

    def test_create_unconnected_asset(self):
        """Tests if an unconnected asset can be created"""
        a = Asset('g1QkoSmSeHdWfGkMKlOlldLn')
        self.assertFalse(a.is_connected())

    # accessing supplied data

    def test_can_access_asset_data(self):
        """Tests if pre-filled data can be accessed"""
        a = Asset.fromdict(self.item_dict)
        self.assertEquals(a.asset_id, 'g1QkoSmSeHdWfGkMKlOlldLn')
        self.assertIsInstance(a.is_favorite, bool)
        self.assertIsInstance(a.videotimestampmodified, datetime.datetime)
        self.assertRaises(Exception, a.some_unexisting_attribute)
        self.assertEqual(a._mmmeta.state, MediaMosaResource.STATE.PARTIAL)

    # accessing unsupplied data

    def test_accessing_empty_asset(self):
        """Tests if an empty asset will automatically fill itself if
        queried"""
        # setup
        self.response.content = open('tests/data/get_asset_id_response.xml')\
                                    .read()
        # test
        asset = Asset('g1QkoSmSeHdWfGkMKlOlldLn', api=self.api)
        # validate
        self.assertIsInstance(asset.is_favorite, bool)
        self.assertNotEquals(self.tt.dump(), '')
        self.tt.clear()
        self.assertIsInstance(asset.videotimestamp, datetime.datetime)
        self.assertEquals(self.tt.dump(), '')
        self.assertRaises(Exception, asset.some_unexisting_attribute)
        self.assertEqual(asset._mmmeta.state, MediaMosaResource.STATE.FULL)

    def test_accessing_partial_asset(self):
        """Tests if a partial asset will automatically fill itself if
        queried"""
        # setup
        self.response.content = open('tests/data/get_asset_id_response.xml')\
                                    .read()
        # test
        asset = Asset.fromdict(self.item_dict, api=self.api, full=False)
        # validate
        self.assertIsInstance(asset.is_favorite, bool)
        self.assertEquals(self.tt.dump(), '')
        self.assertIsInstance(asset.videotimestamp, datetime.datetime)
        self.assertNotEquals(self.tt.dump(), '')
        self.assertRaises(Exception, asset.some_unexisting_attribute)
        self.assertEqual(asset._mmmeta.state, MediaMosaResource.STATE.FULL)



import hashlib
import random
import requests
import xml.sax

from resources import Asset, Mediafile, AssetList
from response import MediaMosaResponseContentHandler


class ApiException(Exception):
    pass


class MediaMosaAPI(object):

    def __init__(self, uri):
        self.uri = uri
        self.session = requests.session()

    ## API

    def authenticate(self, username, secret):
        """Authenticates with the server
        """
        self.username = username
        self.secret = secret

        # get challenge
        challenge = self._login_challenge()
        # send challenge response
        success = self._login_response(challenge)
        self.authenticated = success
        return success

    def asset(self, asset_id):
        """Returns a full asset
        """
        headers, items = self._get('/asset/%s' % asset_id)
        return Asset.fromdict(items[0], api=self, full=True)

    def mediafile(self, mediafile_id):
        """Returns a full mediafile
        """
        headers, items = self._get('/mediafile/%s' % mediafile_id)
        return Mediafile.fromdict(items[0], api=self, full=True)

    def asset_list(self, offset=0, limit=10, **kwargs):
        """Returns a list of partial Assets
        """
        headers, items = self._get('/asset', kwargs.update({
                'offset': offset,
                'limit': limit
            }))
        return AssetList(
            headers,
            [Asset.fromdict(item_dict, api=self) for item_dict in items],
            api=self)

    def play(self, mediafile, user_id='pyUser', response=Mediafile.FORMATS.OBJECT):
        headers, items = self._get('/asset/%s/play' % mediafile.asset_id,
            {'user_id': user_id,
             'mediafile_id': mediafile.id,
             'response': response})
        if items:
            return items[0]
        else:
            return None

    ## APPLICATION LAYER

    def _login_challenge(self):
        """Retrieves a login _login_challenge
        """
        headers, items = self._get('/login', {
            'dbus': 'AUTH DBUS_COOKIE_SHA1 %s' % self.username
        })

        if headers.get('request_result') != 'success':
            raise ApiException("Failed receiving challenge")

        return items[0].get('dbus').split(' ')[-1]

    def _login_response(self, challenge):
        """Performs a login response to a particular challenge
        """
        rand = hashlib.md5(str(random.random())).hexdigest()[:8]
        digest = hashlib.sha1('%s:%s:%s' % (challenge, rand, self.secret))\
                        .hexdigest()
        headers, items = self._post('/login',
            {'dbus': 'DATA %s %s' % (rand, digest)})

        # return true if already logged in
        if headers.get('request_result') != 'success':
            return 'already_identified' in \
                headers.get("request_result_description", [])

        return headers.get('request_result') == 'success'

    def _parse(self, response):
        """Parse a Mediamosa-response.
        """
        handler = MediaMosaResponseContentHandler()
        xml.sax.parseString(response, handler)

        return (handler.headers, handler.items)

    ## HTTP LAYER

    def _get_absolute_uri(self, relative_uri):
        return self.uri + relative_uri

    def _post(self, relative_uri, payload={}):
        """Performs a post call to the api
        """
        response = self.session.post(self._get_absolute_uri(relative_uri),
            data=payload)
        if response.status_code != 200:
            raise ApiException('API returned %s' % response.status_code)

        return self._parse(response.content)

    def _get(self, relative_uri, payload={}):
        """Performs a get call to the api.
        """
        response = self.session.get(self._get_absolute_uri(relative_uri),
            params=payload)

        if response.status_code != 200:
            raise ApiException('API returned %s' % response.status_code)

        return self._parse(response.content)



import unittest

from minimock import mock, TraceTracker, Mock
import requests

from mediamosa.api import MediaMosaAPI, ApiException


class TestLoginFunctions(unittest.TestCase):

    def setUp(self):
        self.url = 'http://video.example.com'
        self.username = 'test_username'
        self.app_id = 5
        self.secret = 'test_secret'

        # mock requests library
        self.tt = TraceTracker()
        self.api = MediaMosaAPI(self.url)
        mock('self.api.session', tracker=self.tt)

        self.response = Mock('requests.Response')
        self.response.status_code = 200
        self.api.session.get.mock_returns = self.response
        self.api.session.post.mock_returns = self.response

    def test_get_absolute_rui(self):
        uri = '/a/relative/url'
        self.assertEquals(self.api._get_absolute_uri(uri), self.url + uri)

    def test_internal_get_call(self):
        # setup
        uri = '/login'
        self.response.content = open(
            'tests/data/login_challenge_response.xml').read()
        # test
        self.api._get(uri)
        # validate
        self.assertTrue(
            self.tt.check("Called "
                + "self.api.session.get('http://video.example.com/login', params={})"))

    def test_internal_post_call(self):
        # setup
        uri = '/login'
        self.response.content = open(
            'tests/data/login_challenge_response.xml').read()
        # test
        self.api._post(uri)
        # validate
        self.assertTrue(
            self.tt.check("Called "
                + "self.api.session.post('http://video.example.com/login', data={})"))

    def test_login_get_challenge(self):
        # setup
        self.api.username = self.username
        self.response.content = open(
            'tests/data/login_challenge_response.xml').read()
        # test
        challenge = self.api._login_challenge()
        # validate
        self.assertEquals(challenge, "92dc06bbb703f14354fdfbede9b62ff9")

    def test_login_get_failed_challenge(self):
        # setup
        self.api.username = self.username
        self.response.content = open(
            'tests/data/login_challenge_invalid_account_response.xml').read()
        # validate
        self.assertRaises(ApiException, self.api._login_challenge)

    def test_login_send_challenge_response(self):
        # setup
        self.api.username = self.username
        self.api.secret = self.secret
        self.response.content = open(
            'tests/data/login_challenge_response_response.xml').read()
        # test
        success = self.api._login_response("92dc06bbb703f14354fdfbede9b62ff9")
        # validate
        self.assertEquals(success, True)

    def test_login_send_challenge_failed_response(self):
        # setup
        self.api.username = self.username
        self.api.secret = self.secret
        self.response.content = open(
            'tests/data/login_invalid_challenge_response_response.xml').read()
        # test
        success = self.api._login_response("92dc06bbb703f14354fdfbede9b62ff9")
        # validate
        self.assertEquals(success, False)

    def test_successful_authentication(self):
        # setup
        mock('self.api._login_challenge', returns='xxx', tracker=self.tt)
        mock('self.api._login_response', returns=True, tracker=self.tt)
        # test
        success = self.api.authenticate(self.username, self.secret)
        # validate
        self.assertEquals(success, True)

    def test_failed_authentication(self):
        # setup
        mock('self.api._login_challenge', returns='xxx', tracker=self.tt)
        mock('self.api._login_response', returns=False, tracker=self.tt)
        # test
        success = self.api.authenticate(self.username, self.secret)
        # validate
        self.assertEquals(success, False)



import unittest

from minimock import Mock, mock, TraceTracker

from mediamosa.resources import Asset, Mediafile, MediaMosaResource
from mediamosa.api import MediaMosaAPI


class TestAssetResource(unittest.TestCase):

    def setUp(self):
        self.api = MediaMosaAPI('http://video.example.com')
        self.tt = TraceTracker()
        mock('self.api.session', tracker=self.tt)
        self.response = Mock('requests.Response')
        self.response.status_code = 200
        self.api.session.get.mock_returns = self.response

        self.partial_dict = {u'is_original_file': u'FALSE',
            u'is_streamable': u'FALSE', u'is_downloadable': u'FALSE',
            u'app_id': u'2', u'transcode_inherits_acl': u'TRUE',
            u'tag': '', u'response_metafile_available': u'TRUE',
            u'mediafile_id_source': u'u2ilZNiHdl7iNUdexL7BcFMY',
            u'asset_id': u'g1QkoSmSeHdWfGkMKlOlldLn',
            u'mediafile_id': u'Md2RgaUEVFhfJMeUIbwPOMei',
            u'transcode_profile_id': u'17',
            u'filename': u'Sintel_Trailer1.1080p.DivX_Plus_HD.mp4',
            u'is_protected': u'FALSE', u'ega_stream_url': '',
            u'file_extension': u'mp4', 'metadata': {
            u'is_inserted_md': u'FALSE', u'fps': u'24', u'bpp': u'0.31',
            u'file_duration': u'00:00:52.20', u'colorspace': u'yuv420p',
            u'container_type': u'mov;mp4;m4a;3gp;3g2;mj2', u'height': u'478',
            u'channels': u'stereo', u'width': u'852', u'sample_rate': u'44100',
            u'filesize': u'20543936', u'audio_codec': u'aac',
            u'video_codec': u'h264', u'is_hinted': u'TRUE',
            u'bitrate': u'3012', u'mime_type': u'video/mp4'},
            u'ega_download_url': '', u'ega_play_url': '', u'tool': u'ffmpeg',
            u'response_plain_available': u'TRUE', u'owner_id': u'krkeppen',
            u'response_object_available': u'TRUE',
            u'created': u'2012-07-05 11:38:14',
            u'changed': u'2012-07-05 11:38:14', u'uri': '',
            u'is_still': u'FALSE', u'command':
            u'audiocodec:libfaac;audiobitrate:128000;audiosamplingrate:44100;audiochannels:2;videocodec:libx264;videopreset_quality:slow;videopreset_profile:baseline;2_pass_h264_encoding:2;videobitrate:800000;qmax:17;size:852x480;maintain_aspect_ratio:yes',
            u'group_id': ''}

    def test_getting_full_asset(self):
        """Test fetching a full asset from the api. The asset is fully
        populated and can connect to the api."""
        # setup
        self.response.content = open('tests/data/get_asset_id_response.xml')\
                                    .read()
        # test
        asset = self.api.asset('g1QkoSmSeHdWfGkMKlOlldLn')
        # validate
        self.assertIsInstance(asset, Asset)
        self.assertEqual(asset._mmmeta.state, MediaMosaResource.STATE.FULL)
        self.assertTrue(asset.is_connected())

    def test_getting_mediafiles_from_asset(self):
        """Test getting mediafiles from the asset. Each mediafile is partial
        and can connect to the api."""
        # setup
        self.response.content = open('tests/data/get_asset_id_response.xml')\
                                    .read()
        asset = Asset.fromdict(self.partial_dict, api=self.api, full=False)
        # test
        mediafiles = asset.list_mediafiles()
        self.assertEqual(len(mediafiles), 3)
        for mediafile in mediafiles:
            self.assertIsInstance(mediafile, Mediafile)
            self.assertEqual(mediafile._mmmeta.state,
                MediaMosaResource.STATE.PARTIAL)
            self.assertTrue(mediafile.is_connected())



import datetime


class MediaMosaResource(object):

    class STATE(object):
        EMPTY = 0
        PARTIAL = 1
        FULL = 2

    def __init__(self, resource_id, api=None):
        """Initializes an empty resource.
        """
        self.id = resource_id
        self.data = {}
        self._mmmeta = self._Meta()
        self._mmmeta.api = api
        self._mmmeta.state = self.STATE.EMPTY

    def __getattr__(self, attr):
        """Looks up an attribute in the data dictionary. It will query
        the api if there is no data available.
        """
        # ignore private attributes
        if attr.startswith('_'):
            raise AttributeError

        # return it if it already exists
        if attr in self.data:
            return self.handle(attr, self.data.get(attr))

        # do a lookup for partials and upgrade to full
        if not self._mmmeta.state == self.STATE.FULL and self.is_connected():
            new_resource = self.fetch_resource_from_api(self.id)
            # add received data to resource and change is_empty state
            self.data = new_resource.data
            self._mmmeta.state = self.STATE.FULL
            # retry lookup
            return self.__getattr__(attr)

    def handle(self, attr, value):
        """Will translate the API value to its python equivalent. This
        depends on the BOOLEANS and DATETIMES tuples defined in the
        concrete MediaMosa resource.
        """
        if attr in self.BOOLEANS:
            return value == u'TRUE'
        elif attr in self.DATETIMES:
            return datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
        else:
            return value

    @classmethod
    def fromdict(cls, dct, api=None, full=False):
        """Creates an MediaMosaResource from a resource description
        received from the MediaMosaAPI."""
        res_id = dct.get(cls.RESOURCE_ID_KEY)
        resource = cls(res_id)
        resource.data = dct
        if full:
            resource._mmmeta.state = cls.STATE.FULL
        else:
            resource._mmmeta.state = cls.STATE.PARTIAL
        resource._mmmeta.api = api
        return resource

    def is_connected(self):
        return self._mmmeta.api is not None

    class _Meta(object):
        state = None
        api = None


class Mediafile(MediaMosaResource):
    RESOURCE_ID_KEY = 'mediafile_id'
    BOOLEANS = (
        'is_original_file', 'is_streamable', 'is_downloadable',
        'response_metafile_available', 'is_protected', 'is_inserted_md',
        'is_hinted', 'response_plain_available', 'response_object_available',
        'is_still',
    )
    DATETIMES = (
        'created', 'changed',
    )

    class FORMATS(object):
        DOWNLOAD = 'download'
        METAFILE = 'metafile'
        OBJECT = 'object'
        STILL = 'still'
        PLAIN = 'plain'
        CUPERTINE = 'cupertino'
        RSTP = 'rstp'
        SILVERLIGHT = 'silverlight'

    def fetch_resource_from_api(self, res_id):
        """Performs the api query necessary to retrieve the mediamosa resource.
        """
        return self._mmmeta.api.mediafile(self.id)

    def __repr__(self):
        return "<mediamosa.resources.Mediafile (%s) %s>" % \
            (self.file_extension, self.id)

    def play(self, format=None):
        play_info = self._mmmeta.api.play(
            self, user_id='pyUser',
            response=format or self.FORMATS.OBJECT)
        if play_info:
            return play_info.get('output')
        return None


class Asset(MediaMosaResource):
    RESOURCE_ID_KEY = 'asset_id'
    BOOLEANS = (
        'is_unappropriate', 'is_favorite', 'has_streamable_mediafiles',
        'granted', 'is_empty_asset', 'asset_property_is_hidden',
        'isprivate', 'is_unappropiate', 'is_external', 'is_protected',
    )
    DATETIMES = (
        'videotimestampmodified', 'videotimestamp',
    )

    def fetch_resource_from_api(self, res_id):
        """Performs the api query necessary to retrieve the mediamosa resource.
        """
        return self._mmmeta.api.asset(self.id)

    def source(self):
        """Returns the mediafile describing the source
        """
        mediafiles = self.list_mediafiles()
        for mediafile in mediafiles:
            if mediafile.is_original_file:
                return mediafile
        return None

    def list_mediafiles(self):
        """Returns a list of mediafiles associated with this asset
        """
        return [Mediafile.fromdict(dct,
            api=self._mmmeta.api, full=False) for dct in self.mediafiles]

    def get_mediafile(self, extension='mp4'):
        """Returns the mediafile with a specific extension. If the
        original is of extension it will only return it if there is no
        re-encoded version with the same extension.
        """
        mediafiles = self.list_mediafiles()
        original = None
        for mediafile in mediafiles:
            if mediafile.is_original_file\
                 and mediafile.file_extension == extension:
                original = mediafile

            elif mediafile.file_extension == extension\
                 and not mediafile.is_original_file:
                return mediafile

        return original

    def __repr__(self):
        return "<mediamosa.resources.Asset %s>" % self.id


class AssetList(list):

    DEFAULT_LIMIT = 10

    def __init__(self, headers, body, api=None):
        self._api = api
        self.headers = headers
        self.body = body
        self._update_location_info()
        # add items to the body
        super(AssetList, self).__init__(body)

    def page_offset(self):
        return self.headers.get('item_offset')

    def page_size(self):
        return self.headers.get('item_count')

    def _update_location_info(self):
        """Updates offset and size information based on headers
        information
        """
        self.offset = self.headers.get('item_offset', 0)
        self.item_size = self.headers.get('item_count', 0)

    def _fetch_page(self, offset, limit=None):
        """Fetches another page from the api
        """
        # query new list of items
        new_asset_list = self._api.asset_list(offset=offset,
            limit=limit or self.DEFAULT_LIMIT)
        # update header info and pointers
        self.headers = new_asset_list.headers
        self._update_location_info()
        # repopulate parent list object
        super(AssetList, self).__init__(new_asset_list.body)

    def __getitem__(self, index):
        # convert index relative to internal list
        relative_index = index - self.offset

        # not an integer
        if type(index) != int:
            raise TypeError("Index not an integer")

        # out of bounds check
        if index < 0 or index >= len(self):
            raise IndexError("Index out of range")

        # check if before or after current page
        if (relative_index < 0) or (relative_index > (self.item_size - 1)):
            # fetch the page containing the index
            self._fetch_page(index)
            # retry
            return self.__getitem__(index)

        # located in current page.
        else:
            return super(AssetList, self).__getitem__(relative_index)

    def __getslice__(self, i, j):
        # call the required slice from the API.
        self._fetch_page(i, j - i)
        return super(AssetList, self).__getslice__(0, j - i)

    def __iter__(self):
        self.index = -1
        return self

    def next(self):
        self.index += 1
        if -1 < self.index < len(self):
            return self[self.index]
        else:
            raise StopIteration

    def __len__(self):
        """Returns the total amount of assets, not simple the loaded ones.
        """
        return int(self.headers.get('item_count_total', 0))



import xml.sax.handler

_formats = {
    "item_count": int,
    "item_count_total": int,
    "item_offset": int,
    "request_query_count": int,
    "request_process_time": float,
}
_dictionaries = {
    "dublin_core": lambda handler:
        _setDictionary(handler._dictionaries[-1], key="dublin_core"),
    "header": lambda handler:
        handler.headers,
    "item": lambda handler:
        _setDictionary(handler.items),
    "mediafile": lambda handler:
        _setDictionary(handler.items[-1]["mediafiles"]),
    "metadata": lambda handler:
        _setDictionary(handler._dictionaries[-1], key="metadata"),
    "qualified_dublin_core": lambda handler:
        _setDictionary(handler._dictionaries[-1], key="qualified_dublin_core"),
    "still": lambda handler:
        _setDictionary(handler.items[-1]["mediafiles"][-1]["stills"]),
}
_lists = {
    "mediafiles": lambda handler:
        handler.items[-1].setdefault(u"mediafiles", []),
    "still": lambda handler:
        handler.items[-1].setdefault("mediafiles", [{}])[-1].setdefault(u"stills", []),
}
_ignore = [
    "items",
    "mediafiles",
    "response",
]


def _setDictionary(container, key=None):
    """Set a dictionary.
    """
    dictionary = {}

    if type(container) == dict:
        container[key] = dictionary
    elif type(container) == list:
        container.append(dictionary)

    return dictionary


class MediaMosaResponseContentHandler(xml.sax.handler.ContentHandler):
    """Content-handler for responses from a MediaMosa-server.
    """

    def __init__(self):
        """Constructor.
        """
        self.headers = {}
        self.items = []

        self._elements = []
        self._dictionaries = []
        self._content = None

    def startElement(self, name, attributes):
        """Handle start-element.
        """
        if name in _lists:
            _lists[name](self)

        if name in _dictionaries:
            dict_ = _dictionaries[name](self)
            self._dictionaries.append(dict_)

        self._elements.append(name)
        self._content = u""

    def endElement(self, name):
        """Handle end-element.
        """
        try:
            if name in _ignore:
                pass
            elif name in _dictionaries:
                self._dictionaries.pop()
            elif self._elements[-2] in _dictionaries:
                format_ = _formats.get(name, None)
                if format_:
                    content = format_(self._content)
                else:
                    content = self._content

                self._dictionaries[-1][name] = content
        except IndexError:
            pass

        self._elements.pop()
        self._content = None

    def characters(self, content):
        """Handle characters.
        """
        if self._elements:
            if self._content is not None:
                self._content += content


from setuptools import setup

with open('README.rst', 'r') as file:
    long_desc = file.read()

version = __import__('mediamosa').get_version()

setup(
    name='mediamosa',
    version=version,
    author='UGent Portaal Team',
    author_email='portaal-tech@ugent.be',
    packages=['mediamosa', 'tests'],
    scripts=[],
    url='https://github.com/UGentPortaal/python-mediamosa',
    license='BSD',
    description='A high-level API interface for MediaMosa.',
    long_description=long_desc,
    install_requires=(
        'requests>=1.0.3'
    ),
    classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Console',
          'Environment :: Web Environment',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: BSD License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
          'Topic :: Multimedia :: Video',
          'Topic :: Software Development :: Libraries',
          'Topic :: Utilities'
          ],
)



import unittest

from minimock import Mock, mock, TraceTracker

from mediamosa.resources import AssetList, Asset
from mediamosa.api import MediaMosaAPI


class TestAssetListResource(unittest.TestCase):

    def setUp(self):
        self.api = MediaMosaAPI('http://video.example.com')
        self.tt = TraceTracker()
        mock('self.api.session', tracker=self.tt)
        self.response = Mock('requests.Response')
        self.response.status_code = 200
        self.api.session.get.mock_returns = self.response

    def _get_asset_list(self, amount, total, offset):
        response = open('tests/data/get_asset_list_response.xml').read()
        asset_item = open('tests/data/asset_item.xml').read()

        asset_items = ''
        for i in xrange(offset, offset + amount):
            asset_items += asset_item.replace('{{item_id}}', str(i))

        return response.replace('{{item_count}}', str(amount))\
                       .replace('{{item_count_total}}', str(total))\
                       .replace('{{item_offset}}', str(offset))\
                       .replace('{{asset_items}}', asset_items)

    def test_getting_paginated_assetlist(self):
        """Test fetching an asset list from the api."""
        # setup
        amount = 10
        total_items = 42
        offset = 0
        self.response.content = self._get_asset_list(
            amount, total_items, offset)
        # test
        asset_list = self.api.asset_list()
        # validate
        self.assertIsInstance(asset_list, AssetList)
        self.assertEqual(len(asset_list), total_items)
        self.assertEqual(asset_list._api, self.api)

    def test_fetch_page(self):
        # setup
        amount = 10
        total_items = 42
        offset = 10  # we have assets 10 to 19
        self.response.content = self._get_asset_list(
            amount, total_items, offset)
        asset_list = self.api.asset_list()

        # test first ten
        self.response.content = self._get_asset_list(
            10, total_items, 0)
        asset_list._fetch_page(0, 10)
        self.assertEqual(len(asset_list), total_items)
        self.assertEqual(asset_list.page_size(), 10)
        self.assertEqual(asset_list.page_offset(), 0)

        # test last two
        self.response.content = self._get_asset_list(
            2, total_items, 40)
        asset_list._fetch_page(40, 10)
        self.assertEqual(len(asset_list), total_items)
        self.assertEqual(asset_list.page_size(), 2)
        self.assertEqual(asset_list.page_offset(), 40)

    def test_getting_assets(self):
        """Test whether AssetList will fetch a new page from api if a
        query is done outside the view
        """
        # setup
        amount = 10
        total_items = 42
        offset = 10  # we have assets 10 to 19
        self.response.content = self._get_asset_list(
            amount, total_items, offset)

        asset_list = self.api.asset_list()

        # test INSIDE current page
        asset = asset_list[10]
        self.assertIsInstance(asset, Asset)
        asset = asset_list[19]
        self.assertIsInstance(asset, Asset)

        # test BEFORE current page
        self.response.content = self._get_asset_list(
            10, total_items, 0)

        asset = asset_list[0]
        self.assertIsInstance(asset, Asset)
        asset = asset_list[9]
        self.assertIsInstance(asset, Asset)

        # test AFTER current page
        self.response.content = self._get_asset_list(
            10, total_items, 15)

        asset = asset_list[20]
        self.assertIsInstance(asset, Asset)

        # test Out of Bounds
        self.assertRaises(Exception, asset_list.__getitem__, -1)
        self.assertRaises(Exception, asset_list.__getitem__, 42)

    def test_iterating_over_paginated_assetlist(self):
        # setup
        total_items = 42
        self.response.content = self._get_asset_list(
            total_items, total_items, 0)
        asset_list = self.api.asset_list()
        # test
        count = 0
        for asset in asset_list:
            count += 1
        # validate
        self.assertEqual(count, total_items)
